export default class Request {

  static using(fetch) {
    return new Request(fetch);
  }

  static browser() {
    return new Request(window.fetch.bind(window));
  }

  constructor(fetch) {
    this._fetch = fetch;
  }

  async get(url, data = null, headers = null) {
    return this._request("GET", url, data, headers);
  }

  async post(url, data = null, headers = null) {
    return this._request("POST", url, data, headers);
  }

  async put(url, data = null, headers = null) {
    return this._request("PUT", url, data, headers);
  }

  async delete(url, data = null, headers = null) {
    return this._request("DELETE", url, data, headers);
  }

  async patch(url, data = null, headers = null) {
    return this._request("PATCH", url, data, headers);
  }

  async _request(method, url, data, headers) {
    const completeHeaders = {
      ...{ "Content-Type": "application/json" }, ...headers
    }
    const options = { method: method, headers: completeHeaders }
    if (data) {
      options.body = JSON.stringify(this._extractData(data));
    }

    const res = await this._fetch(url, options);
    return res.json();
  }

  _extractData(data) {
    if (data.entries
      && typeof data.entries === "function") {
      return Object.fromEntries(data);
    }
    return data;
  }

}
